/****************************************************************************
 **
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and Digia.  For licensing terms and
 ** conditions see http://qt.digia.com/licensing.  For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 2.1 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.LGPL included in the
 ** packaging of this file.  Please review the following information to
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
 ** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
 ** General Public License version 3.0 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.  Please review the following information to
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
 **
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/

#include <unit/libusermetricsoutput/QModelListener.h>
#include <gtest/gtest.h>

using namespace TestsUnitUserMetricsOutput;

void QModelListener::rowsAboutToBeRemovedOrInserted(const QModelIndex & parent,
		int start, int end) {
	int i;
	for (i = 0; start + i <= end; i++) {
		QModelIndex mIndex = m_pModel->index(start + i, 0, parent);
		QVariant var = m_pModel->data(mIndex, Qt::DisplayRole);
		QString str = var.toString();

		EXPECT_EQ(str, m_pAboutToStringlist.at(i));
	}
}

void QModelListener::rowsRemovedOrInserted(const QModelIndex & parent, int,
		int) {
	int i;
	// Can the rows that *are* removed be iterated now ?

	// What about rowsAboutToBeInserted - what will the indices be?
	// will insertRow() overwrite existing, or insert (and conseq. grow the model?)
	// What will the item then contain? empty data?

	// RemoveColumn. Does that also fire the rowsRemoved-family signals?

	for (i = 0; i < m_pExpectedStringlist.size(); i++) {
		QModelIndex mIndex = m_pModel->index(i, 0, parent);
		QVariant var = m_pModel->data(mIndex, Qt::DisplayRole);
		QString str = var.toString();

		//qDebug() << "index: " << i << " start: " << start << "end: " << end;
		EXPECT_EQ(str, m_pExpectedStringlist.at(i));
	}
}
